"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const baseMetadataTransformer_1 = require("./baseMetadataTransformer");
const archiver_1 = require("archiver");
const mime_1 = require("mime");
const unzipper_1 = require("unzipper");
const path_1 = require("path");
const utils_1 = require("../../utils");
const errors_1 = require("../../errors");
class StaticResourceMetadataTransformer extends baseMetadataTransformer_1.BaseMetadataTransformer {
    toMetadataFormat(component) {
        return __awaiter(this, void 0, void 0, function* () {
            const { content, type, xml } = component;
            const writerFormat = { component, writeInfos: [] };
            let contentSource;
            if (yield this.componentIsExpandedArchive(component)) {
                const zip = archiver_1.create('zip', { zlib: { level: 3 } });
                zip.directory(content, false);
                zip.finalize();
                contentSource = zip;
            }
            else {
                contentSource = component.tree.stream(content);
            }
            writerFormat.writeInfos.push({
                source: contentSource,
                output: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
            }, {
                source: component.tree.stream(xml),
                output: path_1.join(type.directoryName, path_1.basename(xml)),
            });
            return writerFormat;
        });
    }
    toSourceFormat(component, mergeWith) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const { xml, content } = component;
            const result = { component, writeInfos: [] };
            if (content) {
                const componentContentType = yield this.getContentType(component);
                const mergeContentPath = (_a = mergeWith) === null || _a === void 0 ? void 0 : _a.content;
                const baseContentPath = this.getBaseContentPath(component, mergeWith);
                // only unzip an archive component if there isn't a merge component, or the merge component is itself expanded
                const shouldUnzipArchive = StaticResourceMetadataTransformer.ARCHIVE_MIME_TYPES.has(componentContentType) &&
                    (!mergeWith || mergeWith.tree.isDirectory(mergeContentPath));
                if (shouldUnzipArchive) {
                    const zipBuffer = yield component.tree.readFile(content);
                    result.writeInfos = yield this.createWriteInfosFromArchive(zipBuffer, baseContentPath);
                }
                else {
                    const extension = this.getExtensionFromType(componentContentType);
                    result.writeInfos.push({
                        source: component.tree.stream(content),
                        output: `${baseContentPath}.${extension}`,
                    });
                }
                result.writeInfos.push({
                    source: component.tree.stream(xml),
                    output: ((_b = mergeWith) === null || _b === void 0 ? void 0 : _b.xml) || component.getPackageRelativePath(path_1.basename(xml), 'source'),
                });
            }
            return result;
        });
    }
    getBaseContentPath(component, mergeWith) {
        var _a;
        const baseContentPath = ((_a = mergeWith) === null || _a === void 0 ? void 0 : _a.content) || component.getPackageRelativePath(component.content, 'source');
        return path_1.join(path_1.dirname(baseContentPath), utils_1.baseName(baseContentPath));
    }
    /**
     * "Expanded" refers to a component whose content file is a zip file, and its current
     * state is unzipped.
     */
    componentIsExpandedArchive(component) {
        return __awaiter(this, void 0, void 0, function* () {
            const { content, tree } = component;
            if (tree.isDirectory(content)) {
                const contentType = yield this.getContentType(component);
                if (StaticResourceMetadataTransformer.ARCHIVE_MIME_TYPES.has(contentType)) {
                    return true;
                }
                throw new errors_1.LibraryError('error_static_resource_expected_archive_type', [
                    contentType,
                    component.name,
                ]);
            }
            return false;
        });
    }
    createWriteInfosFromArchive(zipBuffer, baseDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const writeInfos = [];
            const directory = yield unzipper_1.Open.buffer(zipBuffer);
            for (const entry of directory.files) {
                if (entry.type === 'File') {
                    writeInfos.push({
                        source: entry.stream(),
                        output: path_1.join(baseDir, entry.path),
                    });
                }
            }
            return writeInfos;
        });
    }
    getContentType(component) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield component.parseXml()).StaticResource.contentType;
        });
    }
    getExtensionFromType(contentType) {
        // return registered ext, fallback, or the default (application/octet-stream -> bin)
        return (mime_1.getExtension(contentType) ||
            StaticResourceMetadataTransformer.FALLBACK_TYPE_MAP.get(contentType) ||
            mime_1.getExtension(StaticResourceMetadataTransformer.DEFAULT_CONTENT_TYPE));
    }
}
exports.StaticResourceMetadataTransformer = StaticResourceMetadataTransformer;
StaticResourceMetadataTransformer.ARCHIVE_MIME_TYPES = new Set([
    'application/zip',
    'application/x-zip-compressed',
    'application/jar',
    'application/octet-stream',
]);
StaticResourceMetadataTransformer.DEFAULT_CONTENT_TYPE = 'application/octet-stream';
StaticResourceMetadataTransformer.FALLBACK_TYPE_MAP = new Map([
    ['text/javascript', 'js'],
    ['application/x-javascript', 'js'],
    ['application/x-zip-compressed', 'zip'],
    ['text/x-haml', 'haml'],
    ['image/x-png', 'png'],
    ['text/xml', 'xml'],
]);
//# sourceMappingURL=staticResourceMetadataTransformer.js.map